//-----------------------------------------------------
//File: lcd_spi.c
//Auth: ELECTRONIC ASSEMBLY JM
//DATE: 04-17-2014
//-----------------------------------------------------

//---Includes---
#include "sfr_r825.h"
#include "define.h"

#include "lcd_spi.h"

//-----------------------------------------------------
//Attention, you need to change solder bridges!
//-----------------------------------------------------


static void WriteIns		(char ins);
static void WriteData		(char data);
unsigned char CheckBusy		(void);
static void SPI_put (unsigned int dat);
static unsigned char SPI_get (unsigned char rs);

//--- module global varibles ---



//-----------------------------------------------------
//Func: initDispl()
//Desc: inits Display
//-----------------------------------------------------
void initDispl(void)
{
	unsigned char i;

	//Port init (dd -> datadirection register; 1 output, 0 input)
	ddSID = 1;
	SID = 1;
	ddSOD = 0;
	ddSCLK = 1;
	SCLK = 1;
	ddCS = 1;
	CS = 1;
	
	//init Display
	WriteIns(0x38+3);	//function set european chararacter set
	WriteIns(0x08);	//display off
	WriteIns(0x06);	//entry mode set increment cursor by 1 not shifting display
	WriteIns(0x17);	//Character mode and internel power on (have to turn on internel power to get the best brightness)
	WriteIns(0x01);	//clear display
	WriteIns(0x02);	//return home
	WriteIns(0x0C);	//display on

	ClrDisplay();
	DisplayOnOff(DISPLAY_ON | CURSOR_ON | BLINK_ON);
}

//-----------------------------------------------------
//Func: WriteChar(character)
//Desc: sends a single character to display
//-----------------------------------------------------
void WriteChar (char character)
{
	WriteData(character);
}

//-----------------------------------------------------
//Func: WriteString(string)
//Desc: sends a string to display, must be 0 terminated
//-----------------------------------------------------
void WriteString(_far char * string)
{
	do
	{
		WriteData(*string++);
	}
	while(*string);
}


//-----------------------------------------------------
//Func: SetPostion(postion)
//Desc: set postion
//-----------------------------------------------------
void SetPostion(char pos)
{
	WriteIns(LCD_HOME_L1+pos);
}

//-----------------------------------------------------
//Func: DisplayOnOff(control)
//Desc: use definitions of header file to set display
//-----------------------------------------------------
void DisplayOnOff(char data)
{
	WriteIns(0x08+data);
}

//-----------------------------------------------------
//Func: DefineCharacter(memory postion, character data)
//Desc: stores an own defined character
//-----------------------------------------------------
void DefineCharacter(unsigned char postion, unsigned char *data)
{
	unsigned char i=0;
	WriteIns(0x40+8*postion);

	for (i=0; i<8; i++)
	{
		WriteData(data[i]);
	}
	SetPostion(LINE1);
}
//-----------------------------------------------------
//Func: ClrDisplay
//Desc: Clears entire Display content and set home pos
//-----------------------------------------------------
void ClrDisplay(void)
{
	WriteIns(0x01);
	SetPostion(LINE1);
}

//-----------------------------------------------------
//Func: SetROM
//Desc: Changes the Rom code (CG: 0,1,2,3)
//---------------------------------------------------
void SetROM (unsigned char rom)
{
	
	WriteIns(0x38+ (rom & 0x03));
}

//-----------------------------------------------------
//Func: WriteIns(instruction)
//Desc: sends instruction to display
//-----------------------------------------------------
static void WriteIns(char ins)
{
	CheckBusy();
	
	SPI_put(0x0000 + ins);
}

//-----------------------------------------------------
//Func: WriteData(data)
//Desc: sends data to display
//-----------------------------------------------------
static void WriteData(char data)
{
	CheckBusy();
	
	SPI_put(0x0200 + data);
	
}

//-----------------------------------------------------
//Func: CheckBusy()
//Desc: checks if display is idle
//-----------------------------------------------------
unsigned char CheckBusy(void)
{
	unsigned char readData = 1;

	do
	{
		readData = SPI_get(0); 

	}while(readData&0x80); //check for busyflag

	return readData;
}

//-----------------------------------------------------
//Func: SPI_put()
//Desc: sends data over the spi interface
//-----------------------------------------------------
static void SPI_put (unsigned int dat)
{
	unsigned char i = 9;
	CS = 0;
	
	do
	{
		SCLK = 0;   //Clock puls > 300nsec
		SID = dat >> i;
		SCLK = 1;
	}while(i--);
	
	CS = 1;
}

//-----------------------------------------------------
//Func: SPI_get()
//Desc: receives interface
//-----------------------------------------------------
static unsigned char SPI_get (unsigned char rs)
{
	unsigned char i = 7;
	unsigned char ret = 0;
	CS = 0;
	
	SCLK = 0;
	SID = rs;
	SCLK = 1;
	SCLK = 0;
	SID = 1;   //RW-Bit
	SCLK = 1;
	
	//8 dummy reads
	do
	{
		SCLK = 0;
		SCLK = 1;
	}while(i--);
	
	i=7;
	//read data
	do
	{
		SCLK = 0;
		SCLK = 1;
		ret |= SOD << i;		
	}while(i--);
	
	CS = 1;
	
	return ret;
}





